"""Created on Nov 02 21:24:07 2024"""

__version__ = "0.2.1"
__author__ = "Syed Ali Mohsin Bukhari"
__email__ = "syedali.b@outlook.com"
__license__ = "MIT"
__copyright__ = "2024, Syed Ali Mohsin Bukhari"
__url__ = "https://github.com/syedalimohsinbukhari/pyMultiFit"
__description__ = "A library to fit data with multiple fitters."
